#include <iostream>
#include <fstream>
#include <string>

#define NMAX 510

#define MOD1 666013
#define MOD2 1000003

using namespace std;

int n;
int a[NMAX][NMAX], patrat[NMAX][NMAX];
long long puteri_hash1[NMAX], puteri_hash2[NMAX];
long long hash1[NMAX], hash2[NMAX];

ifstream f("prietene.in");
ofstream g("prietene.out");

void initializeaza() {
	for (int i = 0; i < NMAX; ++i)
	{
		puteri_hash1[i] = puteri_hash2[i] = hash1[i] = hash2[i] = 0;
		for (int j = 0; j < NMAX; ++j) {
			a[i][j] = patrat[i][j] = 0;
		}
	}
}

void citeste() {
	int m, x, y;
	f >> n >> m;
	while (m--) {
		f >> x >> y;
		a[x][y] = 1;
	}
}

void preprop_puteri(long long puteri[], const long long MOD) {
	puteri[1] = 2;
	for (int i = 2; i <= n; ++i) {
		puteri[i] = (puteri[i - 1] * 2) % MOD;
	}
}

void ridica_la_patrat() {
	for (int i = 1; i <= n; ++i) {
		for (int j = 1; j <= n; ++j) {
			patrat[i][j] = 0;
			for (int k = 1; k <= n; ++k) {
				patrat[i][j] += a[i][k] * a[k][j];
			}
		}
	}
}

void calculeza_hash(long long hash[], long long puteri[], long long MOD) {
	for (int i = 1; i <= n; ++i) {
		hash[i] = 0;
		for (int j = 1; j <= n; ++j) {
			if (a[i][j] != 0 || patrat[i][j] != 0) {
				hash[i] = (hash[i] + puteri[j]) % MOD; 
			}
		}
	}	
}

void update_hash(int linie, int coloana, long long hash[], long long puteri[], long long MOD, int sign) {
	long long update_value = sign * puteri[coloana];
	if (sign == -1) {
		update_value += MOD;
	}
	hash[linie] = (hash[linie] + update_value) % MOD;
}

void update_hashes(int x, int y, int sign) {
	update_hash(x, y, hash1, puteri_hash1, MOD1, sign);
	update_hash(x, y, hash2, puteri_hash2, MOD2, sign);
}

void sterge_muchie(int x, int y) {
	a[x][y] = 0;
	for (int z = 1; z <= n; ++z) {
		if (a[y][z] == 1) {
			--patrat[x][z];
			if (patrat[x][z] == 0 && a[x][z] == 0) {
				update_hashes(x, z, -1);
			}
		} 
		if (a[z][x] == 1) {
			--patrat[z][y];
			if (patrat[z][y] == 0 && a[z][y] == 0) {
				update_hashes(z, y, -1);
			}
		} 
	}
	if (patrat[x][y] == 0) {
		update_hashes(x, y, -1);
	}
}

void adauga_muchie(int x, int y) {
	int previous_state_patrat = patrat[x][y];
	a[x][y] = 1;
	for (int z = 1; z <= n; ++z) {
		if (a[y][z] == 1) {
			++patrat[x][z];
			if (patrat[x][z] == 1 && a[x][z] == 0) {
				update_hashes(x, z, 1);
			}
		} 
		if (a[z][x] == 1) {
			++patrat[z][y];
			if (patrat[z][y] == 1 && a[z][y] == 0) {
				update_hashes(z, y, 1);
			}
		} 
	}
	if (patrat[x][y] == 0) {
		update_hashes(x, y, 1);	
	}
}

void afiseaza_vector(int a[][NMAX], int i) {
	cout << n << "\n";
	for (int j = 1; j <= n; ++j) {
		cout << a[i][j] + patrat[i][j] << " ";
	}
	cout << "\n";
}

long long scade_modular(long long descazut, long long scazator, long long MOD) {
	long long rest = descazut - scazator;
	if (rest < 0) {
		rest += MOD;
	}
	return rest;
}

long long scade_pe_ei_insisi(long long value, long long puteri[], long long MOD, int lin, int col) {
	if (patrat[lin][col] + a[lin][col] != 0) {
		value = scade_modular(value, puteri[col], MOD);
	}
	if (patrat[lin][lin] + a[lin][lin] != 0) {
		value = scade_modular(value, puteri[lin], MOD);
	} 
	return value;
}

string intrebare(const int x, const int y) {
	long long hash1_x = scade_pe_ei_insisi(hash1[x], puteri_hash1, MOD1, x, y);
	long long hash1_y = scade_pe_ei_insisi(hash1[y], puteri_hash1, MOD1, y, x);

	long long hash2_x = scade_pe_ei_insisi(hash2[x], puteri_hash2, MOD2, x, y);
	long long hash2_y = scade_pe_ei_insisi(hash2[y], puteri_hash2, MOD2, y, x);

	if (hash1_x == hash1_y && hash2_x == hash2_y) {
		return string("YES");
	}
	return string("NO");
}

void afiseaza_matrice(int a[][NMAX]) {
	cout << n << "\n";
	for (int i = 1; i <= n; ++i) {
		for (int j = 1; j <= n; ++j) {
			cout << a[i][j] + patrat[i][j] << " ";
		}
		cout << "\n";
	}
}

void solve() {
	const string ask = "q";
	const string del = "d";
	string operatie;

	int m, x, y;

	f >> m;
	while (m--) {
		f >> operatie >> x >> y;
		if (operatie == ask) {
			g << intrebare(x, y) << "\n";
		} else if (operatie == del) {
			sterge_muchie(x, y);
		} else {
			adauga_muchie(x, y); 
		}
	}
}

int main() {
	int t;
	f>>t;
	while (t--) {
		initializeaza();

		citeste();

		preprop_puteri(puteri_hash1, MOD1);
		preprop_puteri(puteri_hash2, MOD2);

		ridica_la_patrat();

		calculeza_hash(hash1, puteri_hash1, MOD1);
		calculeza_hash(hash2, puteri_hash2, MOD2);

		solve();
	}
	return 0;
}
